@@ -6,10 +6,6 @@ new_cache = function() {
 
     cache_path = function(hash) {
         d = dirname(hash)
-        if (!is_abs_path(d)) {
-            id = input_dir()
-            d = file.path(id, d)  # turn relative path to abs path
-        }
         if (!file.exists(d)) dir.create(d, showWarnings = FALSE, recursive = TRUE)
         file.path(d, basename(hash))
     }
